function [offspring] = F_reproduction(pop, fitness, finalgen, lower, upper,  L)
%function to peform GP based reproduction
%input:     pop - the current subpopulation
%           fitness - the function values of the current subpopulation
%           finalgen - sign to check if it is the final generation
%           lower - the lower boundary of the decision variables
%           upper - the upper oundary of the decision variables
%           L - random group size
%output:    offspring - the offspring population

M = size(fitness,2);
n = size(pop,2);

%training and sampling, if training data is sufficient
%a minal number of training data in each group (M in total) is 2
%therefore, the minal traning data size (i.e., the population size) is 2*M
if(size(pop,1) >= 2*M)
    
    %f limits (used in sampling)
    flower = min(fitness,[], 1); tlower = min(pop, [], 1);
    fupper = max(fitness,[], 1); tupper = min(pop, [], 1);
    extension = (0.5)*(fupper- flower);
    flower = flower - extension; fupper = fupper + extension;
    
    newpop = [];
    %train M groups of GP models
    for k = 1:M
        %randomly shift the data
        tpopsize = size(pop,1);
        datalist = randperm(tpopsize);
        %select training data for current GP model group
        if(finalgen == 0)
            idx1 = (k - 1)*floor(tpopsize/M) + 1;
            idx2 = k*floor(tpopsize/M);
        else
            %use up all training data if it is the final generation
            idx1 = 1; idx2 = tpopsize;
        end;
        %training data
        trfitness = fitness( datalist(idx1:idx2), :);
        trpop = pop( datalist(idx1:idx2), :);
        %sample data
        PSsample = pop( datalist(idx1:idx2), : );
        samplesize = size(PSsample,1);
        
        
        if(finalgen == 1)
            %random group size if n if it is the final generation
            subsize = n;
        else
            subsize = L;
        end;
        
        
        %perform random grouping based GP training and sampling
        randidx = randperm(n);
        for ii = 1 : subsize
            %generate training data
            i = randidx(ii);
            %trdata = [fitness(:,k) pop(:,i)];
            trdata = [trfitness(:,k) trpop(:,i)];
            
            %sampling
            leftb = flower(k); rightb = fupper(k);
            [PSsample(:,i)] = F_gp(trdata, samplesize, finalgen, leftb, rightb);
            %boundary control
            outidx = [find(PSsample(:,i) < lower(i)); find(PSsample(:,i) > upper(i))];
            PSsample(outidx, i) = tlower(i) + (tupper(i) - tlower(i))*rand(length(outidx), 1);
        end;
        newpop = [newpop; PSsample];
    end;
else
    newpop = pop;
end;

%perform mutation if it is not the final generation
if(finalgen == 0)
    newpop = P_mutation(newpop, upper, lower);
end;

offspring = newpop;

end




